// Yes, I want to use Fisica
import fisica.*;
// Also use the sound library!
import ddf.minim.*;

// Create a master object to maintain the simulation.
FWorld world;
// Two special objects we want to track as we simulate:
// the Prof and the Cannon.  The FProf type and FClicker
// type are defined in two additional tabs that you
// should see in your sketch.  But you don't ever nee
// to look at the code in those tabs if you don't want to.
FProf prof;
FBox cannon;

// Are we currently dragging out a new projectile
// launch?
boolean dragging;
// Multiplicative factor to scale velocity when we
// launch.
float speedNorm = 5.0;

// Create a master object to talk to the sound library.
Minim minim;

void setup()
{
  size(800, 600);
  
  // Initialize the physics engine (as always, we need
  // to tell Fisica about ourselves, via the use of "this").
  Fisica.init(this);
  
  // Initialize the sound engine.
  minim = new Minim( this );

  // Set up the initial world.
  resetWorld();
}

// Create a new physical simulation and overwrite the
// current one.  This way we can always return to the
// starting state of the sketch easily.
void resetWorld()
{  
  // Create a new world, which will hold all the objects
  // being simulated (and if there's already a world that
  // we created previously, just overwriting it with a 
  // fresh one is a convenient way to delete it).
  world = new FWorld();
  
  // Add a ground at the bottom of the sketch.  The easiest
  // way to do that is to add all four walls and then remove
  // the top, left and right.
  world.setEdges();
  world.remove( world.top );
  world.remove( world.left );
  world.remove( world.right );

  // Create an object from which we can launch clickers.
  // For simplicity, just use a square.
  cannon = new FBox( 40, 40 );
  cannon.setFill( 0, 0, 100 );
  cannon.setPosition( 100, height - 150 );
  cannon.setStaticBody( true );
  cannon.setGrabbable( false );
  world.add( cannon );
  
  // TODO: put your tower here, and adjust the Prof's
  // location so that he's sitting on top of it.
  
  prof = new FProf( minim );
  prof.setPosition( 0.8*width, height-43 );
  world.add( prof );
  
  dragging = false;
}

void draw()
{
  // Run the physical simulation for
  // one step
  world.step();
  
  // Draw the world
  background( #c0eeff );
  world.draw();
  
  // Superimpose the power indicator for firing a
  // new clicker, as in the miniputt lab.
  if( dragging ) {
    noFill();
    strokeWeight( 5.0 );
    stroke( 0 );
    line( cannon.getX(), cannon.getY(), mouseX, mouseY );
  }
}

// Fisica will call this function to tell us about collisions
// between bodies in the simulation.  We're only interested
// in collisions that involve the Prof.
void contactResult( FContactResult con )
{
  if( con.getBody1() == prof || con.getBody2() == prof ) {
    // Instead of processing the collision here, tell the
    // Prof object about it.  In particular, tell it how
    // forceful the collision was (in terms of the magic
    // number getNormalImpulse()).
    prof.processCollision( con.getNormalImpulse() );
  }
}

void mousePressed()
{
  // Check whether the click happened within the cannon
  // body.  If so, initiate a new clicker launch.
  FBody pressed = world.getBody( mouseX, mouseY );
  if( pressed == cannon ) {
    dragging = true;
  }
}

void mouseReleased()
{
  if( dragging ) {
    dragging = false;

    // You let go of the mouse while dragging out a new
    // shot.  Generate a new clicker object and fire it.
    FClicker click = new FClicker( 15 );
    click.setPosition( cannon.getX(), cannon.getY() );
    
    // We'll get it a fairly high density so that it can
    // bash the tower, like a cannonball.
    click.setDensity( 5.0 );
    click.setVelocity(
      (cannon.getX() - mouseX)*speedNorm,
      (cannon.getY() - mouseY)*speedNorm );
      
    // Also give it some random initial amount of spin,
    // which will look a bit nicer.
    click.setAngularVelocity( random( -10, 10 ) );
    
    // And add it to the world!
    world.add( click );
  }
}

void keyPressed()
{
  // If you press the space bar, return to the original 
  // state of the world.
  if( key == ' ' ) {
    resetWorld();
  }
}
