// Define a special variation of an FPoly, which has the same
// physical behaviour as a rectangle but that renders itself
// using an image of a Prof.  Do some extra work to create a 
// "proxy" of the shape of the Prof out of a polygon; read the
// coordinates of that polygon out of a text file.  This uses
// object-oriented programming techniques, and you really don't
// need to understand that right now.  In fact, you can ignore
// this file completely, unless you're really interested in
// learning more about his OO programming works ahead of time.

// Use the sound library.
import ddf.minim.*;

class FProf
  extends FPoly
{
  // Store some sound effects
  AudioPlayer[] sounds;
  // And two prof images
  PImage img1;
  PImage img2;
  
  int w;
  int h;
  float sc;
  
  FProf( Minim minim )
  {
    setDensity( 2.0 );
    
    // Load the two Prof images
    img1 = loadImage( "yogi1.png" );
    img2 = loadImage( "yogi2.png" );
    
    sc = 0.75;

    w = img1.width;
    h = img1.height;
    
    // Read in the coordinates of the Prof outline, and
    // use that to initialize the polygon that will represent
    // the Prof outline when looking for collisions.
    String[] strs = loadStrings( "yogi_pts.txt" );
    for( int idx = 0; idx < strs.length; ++idx ) {
      float[] xy = float( split( strs[idx], ' ' ) );
      this.vertex( sc*(4.2*xy[0]-w/2), sc*(h/2-4.2*xy[1]) );
    }
    
    // Load in all the sound effects.
    sounds = new AudioPlayer[6];
    for( int idx = 0; idx < 6; ++idx ) {
      sounds[idx] = minim.loadFile( "snd" + (idx+1) + ".mp3" );
    }
  }
  
  // The main A04 file will let us know when the Prof was
  // involved in a collision.  Use that to decide whether
  // to play a sound effect.
  void processCollision( float power )
  {
    // Don't start a new sound effect if there's already
    // one in progress.
    for( int idx = 0; idx < 6; ++idx ) {
      if( sounds[idx].isPlaying() ) {
        return;
      }
    }
    
    if( power > 20000 ) {
      // Play the special effect for a hard hit.
      sounds[5].rewind();
      sounds[5].play();
    } else if( power > 1000 ) {
      int s = int( random( 5 ) );
      sounds[s].rewind();
      sounds[s].play();
    }
  }
  
  void draw( PGraphics applet )
  {
    // Decide whether to use the alternate Prof image,
    // based on whether he's spinning around a lot, or
    // upside down, or both.
    boolean use2 = false;
    
    if( getAngularVelocity() > 1 ) {
      use2 = true;
    }
    
    float rot = getRotation();
    while( rot < 0 ) {
      rot += TWO_PI;
    }
    rot = rot % TWO_PI;
    
    if( (rot > 0.5*PI) && (rot < 1.5*PI) ) {
      use2 = true;
    }
    
    preDraw( applet );
    if( use2 ) {
      image( img2, -sc*w/2, -sc*h/2, sc*w, sc*h );
    } else {
      image( img1, -sc*w/2, -sc*h/2, sc*w, sc*h );
    }
    postDraw( applet );
    
    // Some debugging code that draws the proxy outline.
    // Use to make sure that the geometry really does 
    // correspond to the image of the Prof.  Not needed
    // once everytyhing is working well.
    
    // setNoFill();
    // super.draw( applet );
  }
}
